@@ -19,14 +19,17 @@
 #ifndef _MN_STOCK_H
 #define _MN_STOCK_H
 
+#include "config.h"
+
 #define MN_STOCK_MAIL			"mail-notification-mail"
-#define MN_STOCK_MAIL_ERROR		"mail-notification-mail-error"
-#define MN_STOCK_NO_MAIL		"mail-notification-no-mail"
-#define MN_STOCK_NO_MAIL_ERROR		"mail-notification-no-mail-error"
 #define MN_STOCK_LOCAL			"mail-notification-local"
 #define MN_STOCK_REMOTE			"mail-notification-remote"
 #define MN_STOCK_UNSUPPORTED		"mail-notification-unsupported"
+#ifdef WITH_GMAIL
 #define MN_STOCK_GMAIL			"mail-notification-gmail"
+#endif
+#define MN_STOCK_SYSTEM_MAILBOX		"mail-notification-system-mailbox"
+#define MN_STOCK_SELECT_ALL		"mail-notification-select-all"
 
 void mn_stock_init (void);
 
